from pdfriend.commands.version import version
from pdfriend.commands.help import help
from pdfriend.commands.merge import merge
from pdfriend.commands.invert import invert
from pdfriend.commands.remove import remove
from pdfriend.commands.swap import swap
from pdfriend.commands.edit import edit
from pdfriend.commands.clear import clear
from pdfriend.commands.weave import weave
from pdfriend.commands.split import split
from pdfriend.commands.encrypt import encrypt
from pdfriend.commands.decrypt import decrypt
from pdfriend.commands.metadata import metadata
